within ;
package planets
  model Gravitation
    extends PlanarMechanics.Interfaces.PartialTwoFrames;
    import SI = Modelica.SIunits;

    parameter StateSelect stateSelect=StateSelect.default
      "Priority to use phi and w as states" annotation(HideResult=true,Dialog(tab="Advanced"));
  //  parameter Modelica.SIunits.RotationalSpringConstant c_phi(final min=0, start=1.0e5)
  //    "Spring constant";
    parameter Modelica.SIunits.Position s_relx0=0 "Unstretched spring length";
    parameter Modelica.SIunits.Position s_rely0=0 "Unstretched spring length";
  //  parameter Modelica.SIunits.Angle phi_rel0=0 "Unstretched spring angle";
    parameter GravitationalConstant G = 1.0;

    Modelica.SIunits.Position s_relx(final stateSelect=stateSelect, start = 0)
      "Spring length" annotation(Dialog(group="Initialization", showStartAttribute=true));
    Modelica.SIunits.Position s_rely(final stateSelect=stateSelect, start = 0)
      "Spring length" annotation(Dialog(group="Initialization", showStartAttribute=true));

     Modelica.SIunits.Position r;

  //  Modelica.SIunits.Angle phi_rel(final stateSelect=stateSelect, start = 0)
  //    "Spring angle" annotation(Dialog(group="Initialization", showStartAttribute=true));
    Modelica.SIunits.Force f_x "Force in x direction";
    Modelica.SIunits.Force f_y "Force in y direction";
    Modelica.SIunits.Force f;

    parameter Modelica.SIunits.Mass m1 "Mass of planet 1";
    parameter Modelica.SIunits.Mass m2 "Mass of planet 2";

    parameter SI.Position s_small = 1.E-10
      "Prevent zero-division if distance between frame_a and frame_b is zero" annotation (Dialog(
        tab="Advanced"));

    parameter Boolean enableAssert = true
      "Cause an assert when the distance between frame_a and frame_b < s_small"
                                                                                annotation (Dialog(
        tab="Advanced"));

    parameter SI.Length zPosition = planarWorld.defaultZPosition
      "Position z of cylinder representing the fixed translation" annotation (Dialog(
        tab="Animation", group="if animation = true", enable=animate));
    parameter Integer numberOfWindings = 5 "Number of spring windings"
      annotation (Dialog(tab="Animation", group="if animation = true", enable=animate));
    input SI.Position width = planarWorld.defaultForceWidth "Width of spring"
      annotation (Dialog(tab="Animation", group="if animation = true", enable=animate));
    input SI.Position coilWidth = width/10 "Width of spring coil"
      annotation (Dialog(tab="Animation", group="if animation = true", enable=animate));
    input Modelica.Mechanics.MultiBody.Types.SpecularCoefficient
      specularCoefficient = planarWorld.defaultSpecularCoefficient
      "Reflection of ambient light (= 0: light is completely absorbed)"
      annotation (Dialog(tab="Animation", group="if animation = true", enable=animate));
    input PlanarMechanics.Types.Color color = PlanarMechanics.Types.Defaults.SpringColor
      "Color of spring"
      annotation (Dialog(tab="Animation", group="if animation = true", enable=animate));

    SI.Length length
      "Distance between the origin of frame_a and the origin of frame_b";
    SI.Position r_rel_0[3]
      "Position vector from frame_a to frame_b resolved in world frame";
    Real e_rel_0[3](each final unit="1")
      "Unit vector in direction from frame_a to frame_b, resolved in world frame";

    //Visualization
    import MB = Modelica.Mechanics.MultiBody;
    parameter Boolean animate = true "Enable animation"
                                                       annotation(Dialog(group="Animation"));
    MB.Visualizers.Advanced.Shape contactA(
      shapeType="cylinder",
      specularCoefficient=specularCoefficient,
      length=0.1,
      width=0.1,
      height=0.1,
      lengthDirection={0,0,1},
      widthDirection={1,0,0},
      r_shape={0,0,-0.06},
      r={frame_a.x,frame_a.y,zPosition},
      R=MB.Frames.nullRotation()) if planarWorld.enableAnimation and animate;
    MB.Visualizers.Advanced.Shape contactB(
      shapeType="cylinder",
      specularCoefficient=specularCoefficient,
      length=0.1,
      width=0.1,
      height=0.1,
      lengthDirection={0,0,1},
      widthDirection={1,0,0},
      r_shape={0,0,-0.06},
      r={frame_b.x,frame_b.y,zPosition},
      R=MB.Frames.nullRotation()) if planarWorld.enableAnimation and animate;
    MB.Visualizers.Advanced.Shape lineShape(
      shapeType="spring",
      color=color,
      specularCoefficient=specularCoefficient,
      length=length,
      width=width,
      height=coilWidth*2,
      lengthDirection=e_rel_0,
      widthDirection= {0,1,0},
      extra=numberOfWindings,
      r={frame_a.x,frame_a.y,zPosition}) if planarWorld.enableAnimation and animate;
  equation

    if enableAssert then

      assert(noEvent(length > s_small), "
The distance between the origin of frame_a and the origin of frame_b
of a Spring component became smaller as parameter s_small
(= a small number, defined in the \"Advanced\" menu). The distance is
set to s_small, although it is smaller, to avoid a division by zero
when computing the direction of the line force. Possible reasons
for this situation:
- At initial time the distance may already be zero: Change the initial
  positions of the bodies connected by this element.
- Hardware stops are not modeled or are modeled not stiff enough.
  Include stops, e.g., stiff springs, or increase the stiffness
  if already present.
- Another error in your model may lead to unrealistically large forces
  and torques that would in reality destroy the stops.
- The flange_b connector might be defined by a pre-defined motion,
  e.g., with Modelica.Mechanics.Translational.Position and the
  predefined flange_b.s is zero or negative.
  ");

    end if;
    r_rel_0 = {s_relx, s_rely, 0};
    length = Modelica.Math.Vectors.length(r_rel_0);
    e_rel_0 = r_rel_0/Modelica.Mechanics.MultiBody.Frames.Internal.maxWithoutEvent(length, s_small);

    //phi_rel = frame_b.phi - frame_a.phi;
    frame_a.t = 0;
    frame_b.t = 0;
    s_relx = frame_b.x-frame_a.x;
    s_rely = frame_b.y-frame_a.y;
    r = sqrt(s_relx^2 + s_rely^2);
    f = G*m1*m2/(r^2);
    f_x = s_relx/r*f;
    f_y = s_rely/r*f;
    //f^2 = f_x^2 + f_y^2;
    //f_x/f_y = s_relx/s_rely;
    frame_a.fx = -f_x;
    frame_b.fx = f_x;
    frame_a.fy = -f_y;
    frame_b.fy = f_y;

    annotation (
      Documentation(revisions="<html><p><img src=\"modelica://PlanarMechanics/Resources/Images/dlr_logo.png\"/> <b>Developed 2010-2014 at the DLR Institute of System Dynamics and Control</b></p></html>",  info="<html>
<p>A <i>linear translational spring</i>. x- and y direction stiffness can be parameterized.</p>
</html>"),
      Icon(coordinateSystem(
          preserveAspectRatio=true,
          extent={{-100,-100},{100,100}},
          grid={2,2}), graphics={
          Line(
            points={{-100,0},{-58,0},{-43,-30},{-13,30},{17,-30},{47,30},{62,0},{100,
                0}},
            thickness=0.5),
          Line(points={{-70,-76},{-70,0}}, color={128,128,128}),
          Line(points={{70,-78},{70,0}}, color={128,128,128}),
          Line(points={{-70,-70},{70,-70}},
                                          color={128,128,128}),
          Polygon(
            points={{60,-67},{70,-70},{60,-73},{60,-67}},
            lineColor={128,128,128},
            fillColor={128,128,128},
            fillPattern=FillPattern.Solid),
          Text(
            extent={{-40,-66},{40,-42}},
            lineColor={128,128,128},
            textString="phi_rel"),
          Text(
            extent={{-150,80},{150,40}},
            textString="%name",
            lineColor={0,0,255}),
          Text(
            extent={{-108,-24},{-72,-49}},
            lineColor={128,128,128},
            textString="a"),
          Text(
            extent={{72,-24},{108,-49}},
            lineColor={128,128,128},
            textString="b")}),
      Diagram(coordinateSystem(
          preserveAspectRatio=true,
          extent={{-100,-100},{100,100}},
          grid={2,2}), graphics={
          Line(
            points={{-100,0},{-58,0},{-43,-30},{-13,30},{17,-30},{47,30},{62,0},{100,
                0}},
            thickness=0.5),
          Line(points={{-68,0},{-68,65}},  color={128,128,128}),
          Line(points={{72,0},{72,65}},  color={128,128,128}),
          Line(points={{-68,60},{72,60}}, color={128,128,128}),
          Polygon(
            points={{62,63},{72,60},{62,57},{62,63}},
            lineColor={128,128,128},
            fillColor={128,128,128},
            fillPattern=FillPattern.Solid),
          Text(
            extent={{-20,40},{20,65}},
            lineColor={0,0,255},
            textString="phi_rel")}));
  end Gravitation;

  type GravitationalConstant = Real (final quantity="GravitationalConstant", final unit=
             "N.m2/kg2");
  model FixedPlanetSystem
    PlanarMechanics.Parts.Body body(
      m=2,
      I=0,
      v(start={0,1}, fixed=true),
      r(start={1,0}, fixed=false))
      annotation (Placement(transformation(extent={{14,0},{34,20}})));
    PlanarMechanics.Parts.Body body1(
      m=1,
      I=0,
      r(start={-1,0}, fixed=true),
      v(start={0,-1}, fixed=true)) annotation (Placement(transformation(
          extent={{-10,-10},{10,10}},
          rotation=180,
          origin={-30,10})));
    inner PlanarMechanics.PlanarWorld planarWorld(g={0,0})
      annotation (Placement(transformation(extent={{-90,72},{-70,92}})));
    PlanarMechanics.Parts.FixedTranslation fixedTranslation(r={2,0})
      annotation (Placement(transformation(extent={{-10,0},{10,20}})));
  equation
    connect(body1.frame_a, fixedTranslation.frame_a) annotation (Line(
        points={{-20,10},{-10,10}},
        color={95,95,95},
        thickness=0.5,
        smooth=Smooth.None));
    connect(body.frame_a, fixedTranslation.frame_b) annotation (Line(
        points={{14,10},{10,10}},
        color={95,95,95},
        thickness=0.5,
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end FixedPlanetSystem;

  model PlanetSystem

    PlanarMechanics.Parts.Body body(
      phi(start=0, fixed=true),
      w(fixed=true, start=0),
      I=1,
      m=10,
      r(start={0,0}, fixed=true),
      v(start={0,-0.2}, fixed=true))
                                   annotation (Placement(transformation(
          extent={{-21,-23},{21,23}},
          rotation=180,
          origin={-73,7})));
    PlanarMechanics.Parts.Body body1(
      phi(start=0, fixed=true),
      w(start=0, fixed=true),
      m=1,
      I=1,
      r(start={2,0}, fixed=true),
      v(start={0,2}, fixed=true))
      annotation (Placement(transformation(extent={{44,-2},{64,18}})));
    inner PlanarMechanics.PlanarWorld planarWorld(g={0,0})
      annotation (Placement(transformation(extent={{-74,64},{-54,84}})));
    Gravitation gravitation(
      m1=1,
      m2=2,
      G=10) annotation (Placement(transformation(extent={{-12,-2},{8,18}})));
  equation
    connect(body.frame_a, gravitation.frame_a) annotation (Line(
        points={{-52,7},{-36,7},{-36,8},{-12,8}},
        color={95,95,95},
        thickness=0.5,
        smooth=Smooth.None));
    connect(body1.frame_a, gravitation.frame_b) annotation (Line(
        points={{44,8},{8,8}},
        color={95,95,95},
        thickness=0.5,
        smooth=Smooth.None));
    annotation (Diagram(graphics));
  end PlanetSystem;
  annotation (uses(Modelica(version="3.2.1"), PlanarMechanics(version=
            "1.4.0 Development Version")));
end planets;
